@@ -6,32 +6,47 @@ import sys
 from typing import Callable, Dict
 from xml import sax
 
-import http_sfv  # type: ignore
+from blessings import Terminal  # type: ignore[import]
+import http_sfv
 
-from .validate import RfcHttpValidator
+from .retrofit import typemap as base_typemap
+from .validate import RfcHttpValidator, ValidatorUi
 
-__version__ = "0.2.1"
+__version__ = "0.2.2"
+
+term = Terminal()
 
 
-class ValidatorCLI:
-    def __init__(self):
+class ValidatorCLI(ValidatorUi):
+    def __init__(self) -> None:
         self.args = self.parse_args()
         self.typemap = self.load_typemap()
+        self.errors = 0
         self.run()
 
     def run(self) -> None:
-        errors = 0
         for fh in self.args.file:
-            handler = RfcHttpValidator(self.typemap, self.status)
+            handler = RfcHttpValidator(self.typemap, self)
             handler.filename = fh.name  # type: ignore
             sax.parse(fh, handler)
-            errors += handler.errors
-        if errors > 0:
+        if self.errors > 0:
             sys.exit(1)
 
-    def status(self, *args):
+    def status(self, message: str) -> None:
         if not self.args.quiet:
-            print(*args)
+            print(message)
+
+    def success(self, subject: str, message: str) -> None:
+        if not self.args.quiet:
+            print(f"{subject} -- {term.green}{message}{term.normal}")
+
+    def error(self, subject: str, message: str) -> None:
+        self.errors += 1
+        print(f"{subject}: {term.red}{message}{term.normal}")
+
+    def skip(self, subject: str, message: str) -> None:
+        if not self.args.quiet:
+            print(f"{subject}: {term.yellow}{message}{term.normal}")
 
     def parse_args(self) -> argparse.Namespace:
         parser = argparse.ArgumentParser(
@@ -73,7 +88,7 @@ class ValidatorCLI:
             "--quiet",
             dest="quiet",
             action="store_true",
-            help="suppress status messages",
+            help="only output errors",
         )
         parser.add_argument(
             "file",
@@ -84,27 +99,14 @@ class ValidatorCLI:
         return parser.parse_args()
 
     def load_typemap(self) -> Dict[str, Callable]:
-        typemap: Dict[str, Callable] = {}
+        typemap = self.process_typemap(base_typemap)
         if self.args.map:
             try:
-                rawmap = json.load(self.args.map)
+                jsonmap = json.load(self.args.map)
             except (IOError, ValueError) as why:
-                sys.stderr.write(f"ERROR loading JSON: {why}\n")
-                sys.exit(1)
-            try:
-                typemap.update(
-                    {
-                        k.lower(): {
-                            "item": http_sfv.Item,
-                            "list": http_sfv.List,
-                            "dict": http_sfv.Dictionary,
-                        }[v]
-                        for (k, v) in rawmap.items()
-                    }
-                )
-            except KeyError as why:
-                sys.stderr.write(f"ERROR loading JSON value: {why}\n")
-                sys.exit(1)
+                self.fatal_error(f"Cannot load JSON: {why}")
+            typemap.update(self.process_typemap(jsonmap))
+
         for item in self.args.item:
             typemap[item.lower()] = http_sfv.Item
         for _list in self.args.list:
@@ -113,6 +115,24 @@ class ValidatorCLI:
             typemap[_dict.lower()] = http_sfv.Dictionary
         return typemap
 
+    def process_typemap(self, typemap: Dict[str, str]) -> Dict[str, Callable]:
+        try:
+            return {
+                k.lower(): {
+                    "item": http_sfv.Item,
+                    "list": http_sfv.List,
+                    "dict": http_sfv.Dictionary,
+                }[v]
+                for (k, v) in typemap.items()
+            }
+        except KeyError as why:
+            self.fatal_error(f"Cannot load field type mapping: {why}")
+            raise
+
+    def fatal_error(self, message: str) -> None:
+        sys.stderr.write(f"{term.red}FATAL ERROR:{term.normal} {message}\n")
+        sys.exit(1)
+
 
 def main() -> None:
     ValidatorCLI()
