@@ -1,124 +1,126 @@
+from os.path import basename
 from typing import Callable, Dict, List
 from xml.sax.handler import ContentHandler
 
 from .methods import REGISTERED_METHODS
 
 
+class ValidatorUi:
+    def status(self, message: str) -> None:
+        pass
+
+    def skip(self, subject: str, message: str) -> None:
+        pass
+
+    def success(self, subject: str, message: str) -> None:
+        pass
+
+    def error(self, subject: str, message: str) -> None:
+        pass
+
+
 class RfcHttpValidator(ContentHandler):
-    def __init__(self, typemap: Dict[str, Callable], status: Callable[[str], None]):
+    def __init__(self, typemap: Dict[str, Callable], ui: ValidatorUi):
         ContentHandler.__init__(self)
         self.typemap = typemap
-        self.status = status
+        self.ui = ui
         self.listening = False
         self.content = ""
-        self.type = None
-        self.errors = 0
+        self.type: str = None
 
-    def startElement(self, name, attrs):
+    def startElement(self, name: str, attrs: Dict[str, str]) -> None:
         if name in ["sourcecode", "artwork"] and "type" in attrs.keys():
             self.listening = True
             self.type = attrs["type"]
 
-    def endElement(self, name):
+    def endElement(self, name: str) -> None:
         if self.listening:
             self.listening = False
             if self.type in ["http-message"]:
                 lines = self.content.strip("\n").split("\n")
                 if len(lines) == 0:
-                    self.validationError("Empty http-message")
+                    self.ui.error(self.location(), "Empty http-message")
                     return
                 lines = self.combine_8792(lines)
                 skip_lines = self.check_start_line(lines[0])
                 try:
                     headers = self.combine_headers(lines[skip_lines:])
                 except ValueError as why:
-                    self.validationError(why)
+                    self.ui.error(self.location(), str(why))
                     return
                 for hname, hvalue in headers.items():
                     header_type = self.typemap.get(hname)
                     if header_type:
+                        subject = f"{hname}: {hvalue}"
                         try:
-                            self.validationStatus(
-                                f"validating field value {hname}: {hvalue}"
-                            )
                             header_type().parse(hvalue.encode("ascii"))
+                            self.ui.success(self.location(subject), "valid")
                         except ValueError as why:
-                            self.validationError(why)
+                            self.ui.error(self.location(subject), str(why))
                     else:
-                        self.validationStatus(
-                            f"skipping field value {hname} (no type information)"
-                        )
+                        self.ui.skip(self.location(hname), "no type information")
             else:
-                self.validationStatus(f"skipping section {self.type}")
+                self.ui.skip(self.location(self.type), "not http-message")
         self.content = ""
 
-    def characters(self, content):
+    def characters(self, content: str) -> None:
         if self.listening:
             self.content += content
 
-    def startDocument(self):
-        self.status(f"* Validating {self.filename}")
-
-    def endDocument(self):
-        self.status(f"* {self.errors} errors.")
-        self.status()
-
-    def validationStatus(self, message: str) -> None:
-        self.status(f"  {message}")
-
-    def validationError(self, message: str) -> None:
-        self.status(f"  ERROR at {self.location()}: {message}")
-        self.errors += 1
-
-    def location(self) -> str:
-        return f"{self.filename}:{self._locator.getLineNumber()}"  # type: ignore
+    def location(self, pinpoint: str = "") -> str:
+        return (
+            f"{basename(self.filename)}:{self._locator.getLineNumber()}"  # type: ignore
+            f"'{pinpoint}'"
+        )
 
     def check_start_line(self, start_line: str) -> int:
         if start_line[0].isspace():
-            self.validationError(f"Start line starts with whitespace: '{start_line}'")
+            self.ui.error(
+                self.location(start_line), "Start line starts with whitespace"
+            )
             return 0
         parts = start_line.split(" ")
         if parts[0][-1] == ":":
             return 0  # it must be a header line
         if "http" in parts[0].lower():
-            self.validationStatus(
-                f"{self.location()}: validating status line {start_line}"
-            )
             if parts[0] != "HTTP/1.1":
-                self.validationError(
-                    f"Status line '{start_line}' doesn't start with 'HTTP/1.1'"
+                self.ui.error(
+                    self.location(start_line),
+                    "Status line doesn't start with 'HTTP/1.1'",
                 )
             elif len(parts) < 3:
-                self.validationError(
-                    f"Status line '{start_line}' isn't 'HTTP/1.1 [status_code] [status_phrase]'"
+                self.ui.error(
+                    self.location(),
+                    f"Status line '{start_line}' isn't 'HTTP/1.1 [status_code] [status_phrase]'",
                 )
             else:
                 if not parts[1].isdigit():
-                    self.validationError(f"Status code {parts[1]} is non-numeric")
+                    self.ui.error(self.location(parts[1]), "Non-numeric status code")
                 elif not 99 < int(parts[1]) < 600:
-                    self.validationError(f"Status code {parts[1]} is out of range")
+                    self.ui.error(self.location(parts[1]), "Status code out of range")
         else:
-            self.validationStatus(
-                f"{self.location()}: validating request line {start_line}"
-            )
             if len(parts) < 3:
-                self.validationError("Request line isn't '[method] [url] HTTP/1.1'")
+                self.ui.error(
+                    self.location(), "Request line isn't '[method] [url] HTTP/1.1'"
+                )
             else:
                 if parts[0] not in REGISTERED_METHODS:
-                    self.validationError(f"Method '{parts[0]}' not recognised")
+                    self.ui.error(self.location(parts[0]), "Method not recognised")
                 if parts[2] != "HTTP/1.1":
-                    self.validationError(
-                        f"Request line '{start_line}' doesn't end with 'HTTP/1.1'"
+                    self.ui.error(
+                        self.location(),
+                        f"Request line '{start_line}' doesn't end with 'HTTP/1.1'",
                     )
                 if len(parts) > 3:
-                    self.validationError(f"Request line '{start_line}' has extra text")
+                    self.ui.error(
+                        self.location(start_line), "Request line has extra text"
+                    )
         return 1
 
     def combine_8792(self, lines: List[str]) -> List[str]:
         if not "NOTE: '\\' line wrapping per RFC 8792" in lines[0]:
             return lines
-        else:
-            lines = lines[2:]
+        lines = lines[2:]
         output = []  # type: List[str]
         continuation = False
         for line in lines:
@@ -136,14 +138,13 @@ class RfcHttpValidator(ContentHandler):
 
     def combine_headers(self, lines: List[str]) -> Dict[str, str]:
         headers = {}  # type: Dict[str, str]
-        prev_name = None
+        prev_name: str = None
         in_body = False
         for line in lines:
             if len(line.strip()) == 0:
                 if not headers:  # a blank line before seeing any headers
-                    raise ValueError(f"Body without headers")
+                    raise ValueError("Body without headers")
                 in_body = True
-                self.validationStatus(f"ignoring HTTP message body")
             if in_body:
                 continue
             if line[0] == " ":
@@ -155,10 +156,10 @@ class RfcHttpValidator(ContentHandler):
                 )
             try:
                 name, value = line.split(":", 1)
-            except ValueError:
-                raise ValueError(f"Non-field line '{line}' in content")
+            except ValueError as why:
+                raise ValueError(f"Non-field line '{line}' in content") from why
             if " " in name:
-                self.validationError(f"Whitespace in field name '{name}'")
+                self.ui.error(self.location(name), "Whitespace in field name")
             name = name.lower()
             value = value.strip()
             if name in headers:
